@@ -63,12 +63,14 @@ class cognet:
           m_vars (list[str], optional): Not implemented yet. Defaults to None.
         """
         if model is not None:
+            # inherit atrributes from model object
             self.qnet = model.myQnet
-            # self.cols = np.array(model.features)
             featurenames, samples = data_obj.format_samples(key)
             samples = pd.DataFrame(samples)
             self.cols = np.array(featurenames)
             self.features = pd.DataFrame(columns=np.array(featurenames))
+            
+            # inherit mutable and immutable variables from model obj
             if any(x is not None for x in [model.immutable_vars, model.mutable_vars]):
                 if model.immutable_vars is not None:
                     self.immutable_vars = model.immutable_vars
@@ -79,6 +81,7 @@ class cognet:
             else:
                 self.mutable_vars = self.features
             
+            # inherit and set class attributes.
             self.samples = pd.DataFrame(samples).replace("nan","").fillna("")
             self.samples.columns = np.array(featurenames)
             self.all_samples = self.samples
@@ -97,7 +100,8 @@ class cognet:
     def load_from_dataformatter(self, 
                                 data_obj,
                                 key):
-        """read in either train or test data, specified by key, from data obj
+        """read in either train or test data, specified by key, from data obj,
+        and inherit other attributes.
 
         Args:
           data_obj (class): instance of dataformatter class
@@ -106,6 +110,7 @@ class cognet:
         Returns:
           featurenames, samples: formatted arrays
         """
+        # inherit attributes from dataformatter object
         featurenames, samples = data_obj.format_samples(key)
         if any(x is not None for x in [self.features, self.samples]):
             print("replacing original features/samples with dataformatter data")
@@ -121,7 +126,7 @@ class cognet:
                   year,
                   features_by_year,
                   samples,
-                  qnet):
+                  Qnet):
         '''load cols, features, samples, and qnet.
 
         Args:
@@ -130,6 +135,7 @@ class cognet:
           samples (str): file of samples for that year.
           Qnet (str): Qnet file location.
         '''
+        # set attributes from given files and data
         self.qnet = load_qnet(qnet)
         self.year = year
         self.cols = np.array((pd.read_csv(features_by_year,
@@ -141,6 +147,7 @@ class cognet:
         self.mutable_vars = [x for x in self.cols]
         #[self.cols].fillna('').values.astype(str)[:]
 
+        # read in samples and initialize related attributes
         self.samples=pd.read_csv(samples)
         self.samples = pd.concat([self.samples,self.features], axis=0)
         self.all_samples = self.samples
@@ -163,6 +170,7 @@ class cognet:
         Args:
           IMMUTABLE_FILE (str): file containing the immutable features/vars
         '''
+        # set mutable and immutable variable attributes 
         if self.cols is None:
             raise ValueError("load_data first!")
         self.immutable_vars = pd.read_csv(IMMUTABLE_FILE,index_col=0).transpose()
@@ -179,8 +187,11 @@ class cognet:
           num_samples (int): Set num of samples to subset, default to None, resets to all samples
           random (bool): take random sample if true, ordered sample if false
         '''
+        # each time function is called, reset samples to use_all_samples
+        # this allows us to call nsamples numerous times 
         self.samples = self.all_samples
-        if num_samples is not None:
+        if self.samples is not None:
+            # if a greater number of sample is selected than available, raise error
             if all(x is not None for x in [num_samples, self.samples]):
                 if num_samples > len(self.samples.index):
                     string = 'The number of selected samples ({}) ' + \
@@ -188,11 +199,14 @@ class cognet:
                     string = string.format(num_samples, len(self.samples.index))
                     raise ValueError(string)
 
+                # if the same number of samples is selected as available, print warning
                 if num_samples == len(self.samples.index):
                     string = 'The number of selected samples ({}) ' + \
                         'is equal to the number of samples ({})!'
                     string = string.format(num_samples, len(self.samples.index))
                     print(string)
+                    
+                # if random is true, return random sample, otherwise return an ordered slice
                 if random:
                     self.samples = self.samples.sample(num_samples)
                 else:
@@ -221,14 +235,15 @@ class cognet:
         Args:
           sample (list[str]): vector of sample, must have the same num of features as the qnet
         '''
+        # if variable is not mutable, set its base frequency to zero 
         MUTABLE=pd.DataFrame(np.zeros(len(self.cols)),index=self.cols).transpose()
-                
+             
         for m in self.mutable_vars:
             MUTABLE[m]=1.0
         mutable_x=MUTABLE.values[0]
         base_frequency=mutable_x/mutable_x.sum()
-
-        # commented out for now for testing using smaller qnet
+        
+        # otherwise, set base frequency weighted by variation weight
         for i in range(len(base_frequency)):
             if base_frequency[i]>0.0:
                 base_frequency[i]= self.variation_weight[i]*base_frequency[i]
@@ -246,13 +261,14 @@ class cognet:
           steps (int): number of steps to qsample
           immutable (bool): are there variables that are immutable?
         '''
-        if all(x is not None for x in [self.mutable_vars, sample]):
-            if immutable == True:
+        # immutable, check that mutable variables have been initialized
+        if immutable == True:
+            if all(x is not None for x in [self.mutable_vars, sample]):
                 return qsample(sample,self.qnet,steps,self.getBaseFrequency(self.samples))
-            else:
-                return qsample(sample,self.qnet,steps)
-        elif self.mutable_vars is None:
-            raise ValueError("load_data first!")
+            elif self.mutable_vars is None:
+                raise ValueError("set mutable and immutable variables first!")
+        else:
+            return qsample(sample,self.qnet,steps)
 
     def set_poles(self,
                   POLEFILE,
@@ -269,8 +285,8 @@ class cognet:
         Args:
           steps (int): number of steps to qsample
           POLEFILE (str): file containing poles samples and features
-          pole_1 (str): column name for first pole to use
-          pole_2 (str): column name for second pole to use
+          pole_1 (str): column name for first pole
+          pole_2 (str): column name for second pole
           mutable (bool): Whether or not to set poles as the only mutable_vars
           VERBOSE (bool): boolean flag prints number of pole features not found in sample features if True
           restrict (bool): boolean flag restricts the sample features to polar features if True
@@ -278,40 +294,41 @@ class cognet:
         '''
         invalid_count = 0
         if all(x is not None for x in [self.samples, self.qnet]):
+            # read and set poles
             poles = pd.read_csv(POLEFILE, index_col=0)
             self.poles=poles.transpose()
             self.polar_features = pd.concat([self.features, self.poles], axis=0).fillna('')
             poles_dict = {}
             for column in poles:
                 p_ = self.polar_features.loc[column][self.cols].fillna('').values.astype(str)[:]
+                # qsample poles to qnet
                 poles_dict[column] = self.qsampling(p_,steps)
             self.poles_dict = poles_dict
             self.pL = self.poles_dict[pole_1]
             self.pR = self.poles_dict[pole_2]
-            # self.pL = list(poles_dict.values())[0]
-            # self.pR = list(poles_dict.values())[1]
             self.d0 = qdistance(self.pL, self.pR, self.qnet, self.qnet)
             
+            # restrict sample columns to polar columns
             if restrict:
-                # restrict sample columns to polar columns
                 cols = [x for x in self.poles.columns if x in self.samples.columns]
                 self.samples=self.samples[cols]
                 self.restricted = True
-                
+              
+            # if poles had been restricted before, unrestrict it and set original
             elif self.restricted:
-                # if poles had been restricted before, unrestrict it
                 self.restricted = False
                 self.samples = self.all_samples
                 if self.nsamples is not None:
                     self.set_nsamples(nsamples, random)
-
-            # for x in self.poles.columns:
-            #     if x not in self.samples.columns:
-            #         invalid_count += 1
-            #         self.samples[x]=''
+            
+            # identify pole features that were excluded due to sample features restriction
+            if VERBOSE:
+                for x in self.poles.columns:
+                    if x not in self.samples.columns:
+                        invalid_count += 1
+                        self.samples[x]=''
 
             self.samples = pd.concat([self.features,self.samples], axis=0).fillna('')
-            #self.all_samples = self.samples
             self.samples_as_strings = self.samples[self.cols].fillna('').values.astype(str)[:]
             
             if mutable:
@@ -340,8 +357,9 @@ class cognet:
         """
         if self.qnet is None:
             raise ValueError("load qnet first!")
-        bp1 = self.getBaseFrequency(sample1)
-        bp2 = self.getBaseFrequency(sample2)
+        #bp1 = self.getBaseFrequency(sample1)
+        #bp2 = self.getBaseFrequency(sample2)
+        # qsample samples
         sample1 = qsample(sample1, self.qnet, nsteps1)#, baseline_prob=bp1)
         sample2 = qsample(sample2, self.qnet, nsteps2)#, baseline_prob=bp2)
         return qdistance(sample1, sample2, self.qnet, self.qnet)
@@ -364,18 +382,18 @@ class cognet:
     def polarDistance(self,
                     i,
                     return_dict=None):
-        """return the distance from a sample to the poles
+        """return the distances from a single sample to the poles
 
         Args:
           i (int): index of sample to take
-          return_dict (dict): dict used for multiple sample function
+          return_dict (dict): dictionary containing multiprocessing results
 
         Returns:
           distances: float, distance from sample to each pole
         """
-        samples_as_strings = self.samples[self.cols].fillna('').values.astype(str)[:]
-        p = samples_as_strings[i]
+        p = self.samples_as_strings[i]
         distances = []
+        # calculate from each pole to the sample, and append to array
         for index, row in self.polar_features[self.cols].iterrows():
             row = row.fillna('').values.astype(str)[:]
             distances.append(self.distance(p, np.array(row)))
@@ -395,17 +413,19 @@ class cognet:
         """
         if all(x is not None for x in [self.samples, self.cols,
                                     self.polar_features]):
+            # init mp.Manager and result dict
             manager = mp.Manager()
             return_dict = manager.dict()
             processes = []
             
+            # init and start mp.Processes for individual samples
             for i in range(len(self.samples)):
                 p = mp.Process(target=self.polarDistance, args=(i, return_dict))
                 processes.append(p)
-
             [x.start() for x in processes]
             [x.join() for x in processes]
 
+            # format and save resulting dict
             pole_names = []
             for index, row in self.polar_features[self.cols].iterrows():
                 pole_names.append(index)
@@ -419,10 +439,11 @@ class cognet:
     def distfunc_line(self,
                     i,
                     return_dict=None):
-        '''compute the dist for a row, or vector of samples
+        '''compute the distance for a single sample from all other samples
 
         Args:
           i (int): row
+          return_dict (dict): dictionary containing multiprocessing results
         
         Return:
           line: float, numpy.ndarray
@@ -453,16 +474,19 @@ class cognet:
           return_dict: dictionary containing multiprocessing results
         """
         if all(x is not None for x in [self.samples, self.features]):
+            # init mp.Manager and result dict
             manager = mp.Manager()
             return_dict = manager.dict()
             processes = []
 
+            # init and start mp.Processes for individual samples
             for i in range(len(self.samples)):
                 p = mp.Process(target=self.distfunc_line, args=(i, return_dict))
                 processes.append(p)
-            
             [x.start() for x in processes]
             [x.join() for x in processes]
+            
+            # format and save resulting dict, and tranpose symmetrical distance matrix 
             result=[x for x in return_dict.values()]
             columns = [i for i in range(len(self.samples))]
             result=pd.DataFrame(result,columns=columns, index=columns).sort_index(ascending=False)
@@ -472,7 +496,7 @@ class cognet:
         else:
             raise ValueError("load data first!")
         
-        return pd.DataFrame(return_dict.copy())
+        return result
     
     def polar_separation(self,
                         nsteps=0):
@@ -484,6 +508,7 @@ class cognet:
         Returns:
           self.polar_matrix: dictionary containing multiprocessing results
         """
+        # vectorize and qsample poles
         polar_arraydata = self.polar_features[self.cols].values.astype(str)[:]
         samples_ = []
         for vector in polar_arraydata:
@@ -491,6 +516,7 @@ class cognet:
             sample = qsample(vector, self.qnet, nsteps, baseline_prob=bp)
             samples_.append(sample)
         samples_ = np.array(samples_)
+        # calculate distance matrix for poles
         self.polar_matrix = qdistance_matrix(samples_, samples_, self.qnet, self.qnet)
         return self.polar_matrix
         
@@ -511,18 +537,21 @@ class cognet:
           EMBED_BINARY (os.path.abspath): path to embed binary
         '''
         if all(x is not None for x in [self.year]):
+            # init file names 
             yr = self.year
             PREF = name_pref
             FILE = infile
-
+            DATAFILE = out_dir + 'data_' +yr
+            EFILE = out_dir + PREF + '_E_' +yr
+            DFILE = out_dir + PREF + '_D_' +yr
+            
+            # set embed binary directory
             if EMBED_BINARY is None:
                 EMBED = pkgutil.get_data("cognet.bin", "__embed__.so") 
             else:
                 EMBED = EMBED_BINARY
-            DATAFILE = out_dir + 'data_' +yr
-            EFILE = out_dir + PREF + '_E_' +yr
-            DFILE = out_dir + PREF + '_D_' +yr
-
+            
+            # embed data files
             pd.read_csv(FILE, header=None).to_csv(DATAFILE,sep=' ',header=None,index=None)
             STR=EMBED+' -f '+DATAFILE+' -E '+EFILE+' -D '+DFILE
             subprocess.call(STR,shell=True)
@@ -563,12 +592,15 @@ class cognet:
                                              distance from the left pole,
                                              and distance between poles, respectively
         """
+        # calculate base distance between two poles
         if pole_1 is not None or pole_2 is not None:
             self.__calc_d0(pole_1, pole_2)
-            
+        
+        # calculate distances between sample and the two poles
         p = self.samples_as_strings[i]
         dR = qdistance(self.pR, p, self.qnet, self.qnet)
         dL = qdistance(self.pL, p, self.qnet, self.qnet)
+        
         ideology_index = (dR-dL)/self.d0
         if return_dict is not None:
             return_dict[i] = [ideology_index, dR, dL, self.d0]
@@ -582,18 +614,21 @@ class cognet:
 
         Args:
           i (int): index of sample
-          return_dict (dict): dict containing results
+          return_dict (dict): dictionary containing multiprocessing results
 
         Returns:
           list[float]: std and max of the distances btwn qsamples
         """
+        # qsample sample num_qsample times
         p = self.samples_as_strings[i]
         Qset = [qsample(p, self.qnet, self.steps) for j in np.arange(self.num_qsamples)]
         Qset = np.array(Qset)
 
+        # calculate qdistance matrix for qsampled samples
         matrix = (qdistance_matrix(Qset, Qset, self.qnet, self.qnet))
         Q = matrix.max()
         Qsd = matrix.std()
+        
         if return_dict is not None:
             return_dict[i] = [Qsd, Q]
         return [Qsd, Q]
@@ -626,18 +661,18 @@ class cognet:
         """
         if all(x is not None for x in [self.samples, self.features,
                                     self.pL, self.pR]):
+            # init vars
             self.num_qsamples = num_qsamples
             self.steps = steps
             if pole_1 != 0 or pole_2 != 1:
                 self.__calc_d0(pole_1, pole_2)
             
-            # testing
-            # pd.DataFrame(self.samples_as_strings).to_csv('examples_results/class_allsamples_2018.csv')
-            
+            # init mp.Manager and result dict
             manager = mp.Manager()
             return_dict = manager.dict()
             processes = []
 
+            # init mp.Processes for individual samples based on type of calulation
             if type == 'ideology':
                 for i in range(len(self.samples)):
                     p = mp.Process(target=self.ideology, args=(i, return_dict))
@@ -651,8 +686,11 @@ class cognet:
             else:
                 raise ValueError("Type must be either dispersion or ideology!")
             
+            # start mp.Processes for individual samples
             [x.start() for x in processes]
             [x.join() for x in processes]
+            
+            # format and save resulting dict
             result=[x for x in return_dict.values()]
             result=pd.DataFrame(result,columns=columns).to_csv(outfile)
 
@@ -679,10 +717,10 @@ class cognet:
             if num_samples is not None:
                 self.set_nsamples(num_samples)
 
-            # read sample data
             if polar_comp:
                 self.set_poles(self.qnet, steps, POLEFILE)
             
+            # calculate polar indices
             polar_features = pd.concat([self.features, self.poles], axis=0)
             self.polar_indices=np.where(polar_features[self.cols].fillna('XXXX').values[0]!='XXXX')[0]
         
@@ -695,23 +733,23 @@ class cognet:
                 sample_index,
                 return_dict=None,
                 MISSING_VAL=0.0):
-        '''compute dissonance for each sample_index, helper function for all_dissonance
+        '''compute dissonance for a single sample, helper function for all_dissonance
         
         Args:
           sample_index (int): index of the sample to compute dissonance
-          return_dict (dict): dict containing results
+          return_dict (dict): dictionary containing multiprocessing results
           MISSING_VAL (float): default dissonance value
           
         Returns: 
-          diss: ndarray containing dissonance for sample
+          diss[self.polar_indices]: ndarray containing dissonance for sample
         '''
         if all(x is not None for x in [self.samples, self.features]):
             s = self.samples_as_strings[sample_index]
             if self.polar_indices is None:
                 self.polar_indices = range(len(s))
 
+            # init vars and calculate dissonance for sample
             Ds=self.qnet.predict_distributions(s)
-            
             diss=np.ones(len(Ds))*MISSING_VAL
             for i in self.polar_indices:
                 if s[i] != '':
@@ -738,17 +776,19 @@ class cognet:
         Returns:
           pandas.DataFrame
         '''
+        # init mp.Manager and result dict
         manager = mp.Manager()
         return_dict = manager.dict()
         processes = []
         
+         # init and start mp.Processes for individual samples
         for i in range(len(self.samples)):
             p = mp.Process(target=self.dissonance, args=(i, return_dict))
             processes.append(p)
-
         [x.start() for x in processes]
         [x.join() for x in processes]
 
+        # format and save resulting dict
         result=[x for x in return_dict.values()]
         if self.polar_indices is not None:
             polar_features = pd.concat([self.features, self.poles], axis=0)
@@ -788,15 +828,20 @@ class cognet:
           allow_all_mutable (bool): whether or not all variables are mutable
           
         Returns:
-          s1,base_frequency,MASKrand,
+          s1,
+          base_frequency,
+          MASKrand,
           np.where(base_frequency)[0],
           np.mean(rnd_match_prob),
           np.mean(max_match_prob),
-          s_rand
+          random_sample
         '''
         if self.samples is not None:
+            # init random mutable variable masking
             s0=s.copy()
             s0=np.array(s0)   
+            # double check, because code seems to imply that masking happens in order,
+            # i.e. limited to the first 100 features, if there are only 100 mutable features
             MUTABLE=pd.DataFrame(np.zeros(len(self.cols)),index=self.cols).transpose()
             WITHVAL=[x for x in self.cols[np.where(s0)[0]] if x in self.mutable_vars ]
             MASKrand=[x for x in WITHVAL if random.random() < mask_prob ]
@@ -810,17 +855,19 @@ class cognet:
             #     return np.nan,np.nan,np.nan
             #     return self.getMaskedSample(s)
 
+            # mask sample according to masking (base_frequency)
             s1=s.copy()
             for i in range(len(base_frequency)):
                 if base_frequency[i]>0.0001:
                     s1[i]=''
-                    
-            s_rand=np.copy(s)
+                
+            # create a random sample to test reconstruction effectiveness
+            random_sample=np.copy(s)
             rnd_match_prob=[]        
             max_match_prob=[]        
             D=self.qnet.predict_distributions(s)
             for i in MASKrand:
-                s_rand[np.where(
+                random_sample[np.where(
                     self.cols==i)[0][0]]=self.__choose_one(
                         self.D_null[np.where(self.cols==i)[0][0]].keys())
                 rnd_match_prob=np.append(rnd_match_prob,1/len(
@@ -829,33 +876,55 @@ class cognet:
                     max_match_prob,np.max(
                         list(D[np.where(
                             self.cols==i)[0][0]].values())))
-                
+            
+            # calculate base_frequency if all variables are mutable
             if allow_all_mutable:
-                for m in mutable_vars:
+                WITHVAL=[x for x in self.cols[np.where(s0)[0]]]
+                MASKrand=[x for x in WITHVAL if random.random() < mask_prob ]
+                for m in MASKrand:
                     MUTABLE[m]=1.0
                 mutable_x=MUTABLE.values[0]
                 base_frequency=mutable_x/mutable_x.sum()
+                s1=s.copy()
+                for i in range(len(base_frequency)):
+                    if base_frequency[i]>0.0001:
+                        s1[i]=''
 
             return s1,base_frequency,MASKrand,np.where(
-                base_frequency)[0],np.mean(rnd_match_prob),np.mean(max_match_prob),s_rand
+                base_frequency)[0],np.mean(rnd_match_prob),np.mean(max_match_prob),random_sample
         else:
             raise ValueError("load_data first!")
 
     def randomMaskReconstruction(self,
                                 index=None,
                                 return_dict=None,
-                                sample=None):
+                                sample=None,
+                                index_colname="feature_names",
+                                output_dir="recon_results/",
+                                file_name="recon_tmp.csv"):
         """reconstruct the masked sample by qsampling and comparing to original
         set self.mask_prob and self.steps if needed
 
         Args:
-          return_dict (dict): dict containing results. Defaults to None.
-          sample (list[str], optional): sample vector, must have the same num of features as the qnet. Defaults to None.
           index (int): index of sample to take. Defaults to None.
+          return_dict (dict): dictionary containing multiprocessing results. Defaults to None.
+          sample (list[str], optional): sample vector, must have the same num of features as the qnet. Defaults to None.
+          index_colname (str): column name for index. Defaults to "feature_names"
+          output_dir (str): directory name for output files. Defaults to "recon_results/".
+          file_name (str): base file name for output files Defaults to "recon_tmp.csv".
 
         Raises:
           ValueError: Neither sample or index were given
           ValueError: Both sample and index were given
+          
+        Returns:
+          (1 - (dqestim/dactual))*100,
+          rmatch_u,
+          rmatch,
+          s,
+          qs,
+          random_sample,
+          mask_
         """
         if all(x is None for x in [sample, index]):
             raise ValueError("Must input either sample or index!")
@@ -865,24 +934,33 @@ class cognet:
             s=np.array(pd.DataFrame(sample).fillna('').values.astype(str)[:])
         elif index is not None:
             s=self.samples_as_strings[index]
-            
-        s1,bp,mask_,maskindex,rmatch_u,rmatch,s_rand=self.getMaskedSample(s, 
+        
+        # calculate masked sample and get variables
+        s1,bp,mask_,maskindex,rmatch_u,rmatch,random_sample=self.getMaskedSample(s, 
                                                                         mask_prob=self.mask_prob)
+        # if base_frequency is nan, set return_dict to nans
         if np.isnan(bp).any():
             return_dict[index] = np.nan,np.nan,np.nan
             return np.nan,np.nan,np.nan
+        
+        # make directories
+        if not os.path.exists(output_dir):
+            os.mkdir(output_dir)
 
+        # qsample sample and calculate distances between original and qsampled 
         qs=qsample(s1,self.qnet,self.steps,bp)
-
         dqestim=qdistance(s,qs,self.qnet,self.qnet)
         dactual=qdistance(s,s1,self.qnet,self.qnet)
-        qdistance_time_end = time.time()
-
-        cmpf=pd.DataFrame([s,qs,s_rand],columns=self.cols,index=['s','q','r'])[mask_].transpose()
-        cmpf.index.name='gssvar'
-        cmpf.to_csv('examples_results/CMPF_2018/CMPF-'+str(index)+'.csv')
-        return_dict[index] = (1 - (dqestim/dactual))*100,rmatch_u,rmatch
-        return (1 - (dqestim/dactual))*100,rmatch_u,rmatch,s,qs,s_rand,mask_
+        
+        # format and save sample and qsample statistics and values
+        cmpf=pd.DataFrame([s,qs,random_sample],
+                          columns=self.cols,
+                          index=['sample','qsampled','random_sample'])[mask_].transpose()
+        cmpf.index.name= index_colname
+        file_name = file_name.replace("tmp", str(index))
+        cmpf.to_csv(output_dir+file_name)
+        return_dict[index] = (1 - (dqestim/dactual))*100,rmatch_u,rmatch,s,qs,random_sample,mask_
+        return (1 - (dqestim/dactual))*100,rmatch_u,rmatch,s,qs,random_sample,mask_
 
     def randomMaskReconstruction_multiple(self,
                                           out_file):
@@ -894,28 +972,21 @@ class cognet:
         Returmns:
           result.rederr.mean(), result.rand_err.mean(): mean of reconstruction error and random error
         '''
+        # init mp.Manager and result dict
         manager = mp.Manager()
         return_dict = manager.dict()
         processes = []
         
+        # init and start mp.Processes for individual samples
         for i in range(len(self.samples)):
             p = mp.Process(target=self.randomMaskReconstruction, args=(i, return_dict))
             processes.append(p)
-
         [x.start() for x in processes]
         [x.join() for x in processes]
         
-        #result=pd.DataFrame(return_dict.items())[1]#, columns=['sample','rederr','r_prob','rand_err','s','q','r'])
+        # format and save resulting dict
         result=[x for x in return_dict.values() if isinstance(x, tuple)]
-        # #result=pd.DataFrame(result.tolist())
-        # print(result)
-        # cmprdf=result[[3,4,5]]
-        # mask_=result[[6]]
-        # cmprdf.columns=['s','q','r']#[mask_].transpose()
-        # cmprdf.to_csv("examples_results/CMPF_"+"tmp"+".csv")
-        # print(cmprdf)
-        # result=result[[0,1,2]]
-        result=pd.DataFrame(result,columns=['rederr','r_prob','rand_err'])
+        result=pd.DataFrame(result,columns=['rederr','r_prob','rand_err','sample','qsampled','random_sample','mask_'])
         result.rederr=result.rederr.astype(float)
 
         if self.poles is not None:
@@ -926,22 +997,41 @@ class cognet:
         return pd.DataFrame(return_dict.copy())
     
     def dmat_filewriter(self,
-                        pyfile,
                         QNETPATH,
+                        pyfile="cognet_qdistmatrix.py",
                         MPI_SETUP_FILE="mpi_setup.sh",
                         MPI_RUN_FILE="mpi_run.sh",
-                        MPI_LAUNCHER_FILE="mpi_launcher.sh",
+                        MPI_LAUNCHER_FILE="launcher.sh",
                         YEARS='2016',
                         NODES=4,
                         T=12,
                         num_samples=None,
                         OUTFILE='tmp_distmatrix.csv',
                         tmp_samplesfile="tmp_samples_as_strings.csv"):
-        if all(x is not None for x in [self.poles_dict,self.features,
+        """generate files to compute qdistance matrix using mpi parallelization
+
+        Args:
+          QNETPATH (str): Qnet filepath
+          pyfile (str, optional): Name of generated python file. Defaults to "cognet_qdistmatrix.py".
+          MPI_SETUP_FILE (str, optional): Name of mpi setup script. Defaults to "mpi_setup.sh".
+          MPI_RUN_FILE (str, optional): Name of mpi run script. Defaults to "mpi_run.sh".
+          MPI_LAUNCHER_FILE (str, optional): Launcher script filepath. Defaults to "launcher.sh".
+          YEARS (str, optional): If looping by year, not currently implemented. Defaults to '2016'.
+          NODES (int, optional): Number of nodes to use. Defaults to 4.
+          T (int, optional): Number of hours to reserve nodes for. Defaults to 12.
+          num_samples ([type], optional): How many samples to take. Defaults to None.
+          OUTFILE (str, optional): CSV File to write computed qdist matrix. Defaults to 'tmp_distmatrix.csv'.
+          tmp_samplesfile (str, optional): CSV File to write samples as strings. Defaults to "tmp_samples_as_strings.csv".
+
+        Raises:
+            ValueError: load data if qnet, features, or samples are not present]
+        """
+        if all(x is not None for x in [self.samples,self.features,
                                        self.qnet, self.cols]):
             if num_samples is not None:
                 self.set_nsamples(num_samples)
             
+            # init and make tmp dir 
             tmp_path = "mpi_tmp/"
             if not os.path.exists(tmp_path):
                 os.makedirs(tmp_path)
@@ -949,6 +1039,8 @@ class cognet:
             pd.DataFrame(self.samples_as_strings).to_csv(tmp_path+tmp_samplesfile, header=None, index=None)
             
             w = self.samples.index.size
+            
+            # writing python file
             with open(tmp_path+pyfile, 'w+') as f:
                 f.writelines(["from mpi4py.futures import MPIPoolExecutor\n",
                               "import numpy as np\n",
@@ -981,7 +1073,8 @@ class cognet:
 	                          "\tresult = result.to_numpy()\n",
                               "\tresult = pd.DataFrame(np.maximum(result, result.transpose()))\n"
                               "\tresult.to_csv(\'{}\',index=None,header=None)".format(OUTFILE)])
-                
+            
+            # writing MPI setup file
             with open(tmp_path+MPI_SETUP_FILE, 'w+') as ms:
                 ms.writelines(["#!/bin/bash\n",
                                "YEAR=$1\n\n",
@@ -1009,6 +1102,7 @@ class cognet:
                                "echo \"date; mpiexec -n \"$NUMPROC\" python3 -m mpi4py.futures {}; date\"  >> $PROG\n".format(pyfile),
                                 ])
 
+            # writing MPI run file
             with open(tmp_path+MPI_RUN_FILE, 'w+') as mr:
                 mr.writelines(["#!/bin/bash\n",
                                "YEARS=\'{}\'\n".format(YEARS),
@@ -1017,7 +1111,7 @@ class cognet:
                                "# time requested\n",
                                "T={}\n".format(T),
                                "NUM=\'all\'\n",
-                               "LAUNCH=\'../mpi_launcher.sh\'\n\n",
+                               "LAUNCH=\'{}\'\n\n".format(MPI_LAUNCHER_FILE),
                                "for yr in `echo $YEARS`\n",
                                "do\n",
                                "\techo $yr\n",
@@ -1025,5 +1119,7 @@ class cognet:
                                "\t$LAUNCH -P tmp_\"$yr\" -F -T $T -N \"$NODES\" -C 28 -p broadwl -J ACRDALL_\"$yr\" -M 56\n",
                                "done\n",
                                "rm tmp_\"$yr\"*\n"])
+            os.system("cp {} {}".format(MPI_LAUNCHER_FILE,tmp_path+'mpi_launcher.sh'))
+        
         else:
             raise ValueError("load data first!")
